@@ -9,10 +9,10 @@ import pathlib
 import sys
 import re
 from columnize import columnize
+from mathics_scanner import replace_unicode_with_wl, named_characters
 from mathics.core.expression import Expression, String, Symbol
 from mathics.core.expression import strip_context, from_python
 from mathics.core.rules import Rule
-from mathics.core.characters import named_characters
 
 from pygments import highlight, lex
 from mathicsscript.mmalexer import MathematicaLexer
@@ -62,48 +62,21 @@ CONFIGDIR = os.path.join(CONFIGHOME, "mathicsscript")
 os.makedirs(CONFIGDIR, exist_ok=True)
 
 try:
-    HISTSIZE = int(os.environ.get("MATHICSSCRIPT_HISTSIZE", 50))
+    HISTSIZE = int(os.environ.get("MATHICSSCRIPT_HISTSIZE"))
 except:
     HISTSIZE = 50
 
-# This doesn't work: investigate
-# HISTFILE = os.path.join(CONFIGDIR, "history")
-HISTFILE = osp.expanduser("~/.mathicsscript_hist")
+HISTFILE = os.path.join(CONFIGDIR, "history")
+
+# Create HISTFILE if it doesn't exist already
+if not os.path.isfile(HISTFILE):
+    pathlib.Path(HISTFILE).touch()
 
 RL_COMPLETER_DELIMS_WITH_BRACE = " \t\n_~!@#%^&*()-=+{]}|;:'\",<>/?"
 RL_COMPLETER_DELIMS = " \t\n_~!@#%^&*()-=+[{]}\\|;:'\",<>/?"
 
-wl_replace_dict = {
-    "": "Ạ",
-    "": "ạ",
-    "": "Ḅ",
-    "": "ḅ",
-    # ...
-    "": "→",
-    "": "↔",
-}
 
-wl_replace_dict_esc = dict((re.escape(k), v) for k, v in wl_replace_dict.items())
-wl_replace_pattern = re.compile("|".join(wl_replace_dict_esc.keys()))
-
-unicode_replace_dict = {v: k for k, v in wl_replace_dict.items()}
-unicode_replace_dict_esc = dict(
-    (re.escape(k), v) for k, v in unicode_replace_dict.items()
-)
-unicode_replace_pattern = re.compile("|".join(unicode_replace_dict_esc.keys()))
-
-
-from mathics.core.parser import LineFeeder
-
-
-def replace_unicode_to_wl(unicode_input: str) -> str:
-    """WL uses some non-unicode character for various things.
-    Replace the unicode equivalent with the WL equivalent.
-    Formal values like FormalA are like this.
-    """
-    return unicode_replace_pattern.sub(
-        lambda m: unicode_replace_dict_esc[re.escape(m.group(0))], wl_input
-    )
+from mathics.core.parser import MathicsLineFeeder
 
 
 def is_pygments_style(style):
@@ -114,7 +87,12 @@ def is_pygments_style(style):
     return True
 
 
-class TerminalShell(LineFeeder):
+class ShellEscapeException(Exception):
+    def __init__(self, line):
+        self.line = line
+
+
+class TerminalShell(MathicsLineFeeder):
     def __init__(
         self,
         definitions,
@@ -154,7 +132,10 @@ class TerminalShell(LineFeeder):
                         inputrc = (
                             "inputrc-unicode" if use_unicode else "inputrc-no-unicode"
                         )
-                        read_init_file(str(parent_dir / inputrc))
+                        try:
+                            read_init_file(str(parent_dir / inputrc))
+                        except:
+                            pass
 
                     parse_and_bind("tab: complete")
                     self.completion_candidates = []
@@ -228,7 +209,7 @@ class TerminalShell(LineFeeder):
         self.definitions.set_attribute(
             "Settings`PygmentsStylesAvailable", "System`Locked"
         )
-        self.definitions.set_attribute("Settings`UseUnicode", "System`Locked")
+        self.definitions.set_attribute("Settings`$UseUnicode", "System`Locked")
 
     def change_pygments_style(self, style):
         if style == self.pygments_style:
@@ -267,24 +248,27 @@ class TerminalShell(LineFeeder):
 
     def read_line(self, prompt):
         if self.using_readline:
-            return self.rl_read_line(prompt)
-        return replace_unicode_to_wl(input(prompt))
+            line = self.rl_read_line(prompt)
+        else:
+            line = input(prompt)
+        if line.startswith("!") and self.lineno == 0:
+            raise ShellEscapeException(line)
+        return replace_unicode_with_wl(line)
 
     def print_result(self, result, output_style=""):
         if result is None:
             # FIXME decide what to do here
             return
 
-        # FIXME: after Mathics last_eval is fixed,
-        # we need only the True branch of this code
-        if hasattr(result, "last_eval"):
-            last_eval = result.last_eval
-            eval_type = last_eval.get_head_name()
-        else:
-            eval_type = ""
-            last_eval = result.result
+        last_eval = result.last_eval
 
         if last_eval is not None:
+            try:
+                eval_type = last_eval.get_head_name()
+            except:
+                print(sys.exc_info()[1])
+                return
+
             out_str = str(result.result)
             if eval_type == "System`Graph":
                 out_str = "-Graph-"
@@ -305,7 +289,10 @@ class TerminalShell(LineFeeder):
                     print(list(lex(out_str, mma_lexer)))
                 out_str = highlight(out_str, mma_lexer, self.terminal_formatter)
             output = self.to_output(out_str)
-            print(self.get_out_prompt("") + output + "\n")
+            if output_style == "text":
+                print(output)
+            else:
+                print(self.get_out_prompt("") + output + "\n")
 
     def rl_read_line(self, prompt):
         # Wrap ANSI color sequences in \001 and \002, so readline
